eval 'exec perl -S -w $0 ${1+"$@"}'
        if 0;

################################################
#
# PLAYRANDOM
# (C) 2000 by
#  LaLa <lala@c64.org>
#
# Requires:
#
#  - Perl5
#  - Un*x
#
# Usage:
#
#  playrandom [-h] [-r regexp] [-l loop] [-s loop] [-c mm:ss|sec] [-p filename]
#
#  Perl script to play a random SID file from HVSC under Un*x.
#
# TODO:
#
# - Allow more than 1 regular expression (drop -r option!?)
#
################################################

# Modify $SIDPLAY if sidplay is not in your PATH.
#
# NOTE: Make sure that you put the *FULL* path here, without any
# environment variables, otherwise this script will not be able
# to kill -9 the sidplay process.

# $SIDPLAY = '/home/imre/bin/sidplay';

# Modify $STILVIEW if STILView is not in your PATH.

$STILVIEW = '/home/imre/bin/stilview';

# Modify $SIDPLAY_OPTIONS if you want to pass in different options.

# $SIDPLAY_OPTIONS = '-f22000 -s -pc';
$SIDPLAY_OPTIONS = '-f44000 -s -pc';

require 5;

use IO::Handle; # Required for flushing STDIN.
use Getopt::Std;

################################################
#
# SUBS
#
################################################

# Function to pretty print.

sub printspec {
    my $text = $_[0];
    my $maxlength = 50;
    my $filler = '-';
    my $i;

    $i = int(($maxlength-length($text)-2)/2);

    print "\e[1m";
    print $filler x $i;
    print "\e[7m $text \e[0m\e[1m";

    $i = ($maxlength-$i-length($text)-2);

    print $filler x $i;
    print "\e[0m\n";
}

# Help stuff

sub printhelp {
    print <<"HELP";
playrandom.pl [-h] [-r regexp] [-l loop] [-s loop] [-c mm:ss|sec] [-p filename]
Plays randomly selected SID tunes from HVSC.
-h : this help
-r : restricts random selection only to those SIDs whose HVSC-relative
     pathnames matches the given Perl regular expression. See examples below.
-l : loop long looping tunes this many times (default: $loopLong)
-s : loop short looping tunes this many times (default: $loopShort)
-c : cutoff time (in mm:ss or sec format) between short and long tunes
     (to determine what is a long and what is a short tune) (default: $cutoffTime sec)
-p : create playlist of the tunes played so far with the given filename. The
     filename must have .spl (SIDPlay/Win) or .m3u (Winamp) as its extension!
     The file is appended as more and more tunes are being played.

\$HVSC_BASE should be set to point to your HVSC base directory, for example:
HVSC_BASE=/home/imre/media/SID/C64Music

\$SIDPLAY should be set to the location of the sidplay executable,
or sidplay should be in your \$PATH.
(SIDPLAY options are set in this script in \$SIDPLAY_OPTIONS!)

\$STILVIEW may be set to the location of the STILView executable,
or STILView may be in your \$PATH,
or don\`t set it at all - STILView is optional.

Examples:
 playrandom.pl => plays randomly selected SID tunes from HVSC.
 playrandom.pl -r ^/Hubbard_Rob/C -p myplaylist.spl => plays tunes from SID
   files in Hubbard\`s dir whose filenames start with a capital C, and creates
   a SIDPlay/Win playlist from the tunes actually played.
 playrandom.pl -r /Hubbard_Rob/Commando.sid -l 2 -c 0:30 -s 1 => plays random
   tunes from this SID file, playing the longer than 30 sec tunes twice and the
   shorter ones only once.
HELP
    exit();
}

# Read list of SIDs and song-length infos from Mike's generated file.

sub readSIDList {
    my $tempFile;
    my $HVSIDSLST;
    my $HVSC_version;

    $tempFile = $basedir . "/DOCUMENTS/hv_sids.txt";

    open (TEMP, "< $tempFile") || die "ERROR: Could not open $tempFile!\n";

    while (<TEMP>) {
        if (/^\s*release (\d+\.\d+)\s*$/) {
            $HVSC_version = $1;
            last;
        }
    }

    close (TEMP);

    if (!$HVSC_version) {
        die "ERROR: Couldn't determine HVSC version number from $tempFile!\n";
    }

    $HVSC_version =~ s/\./_/;
    $HVSIDSLST = $basedir . "/DOCUMENTS/Songlengths.txt";

    open (HVSIDSLST, "< $HVSIDSLST") || die "ERROR: Couldn't open $HVSIDSLST!\n";

    print "Processing $HVSIDSLST...\n\n";

    while (<HVSIDSLST>) {
        if (/(\/\S+\.sid)/) {
            $filename = $1;
            $songlengthlist = <HVSIDSLST>;
            $songlengthlist =~ s/^.+=//;

            # -r option restricts random selection to a given dir.
            # We won't even process those that don't match.
            next if ($filename !~ /$restrict/);

            $i=0;
            foreach $songlength (split(/\s+/, $songlengthlist)) {
                $hvscList{$filename}[$i++] = $songlength;
            }

            # Debug
#            print "$filename " . scalar(@{$hvscList{$filename}}) . " ";
#            print @{$hvscList{$filename}};
#            print "\n";
        }
    }

    close (HVSIDSLST);

    # Sort case-insensitively.
    @hvscList = sort {lc($a) cmp lc($b)} (keys %hvscList);

    printf "Number of SID files in random pool: " . scalar(@hvscList) . "\n\n";
    sleep(1);
}

################################################
#
# MAIN PROGRAM STARTS HERE
#
################################################

# Default settings.
$loopLong = 1;
$loopShort = 2;
$cutoffTime = 30;
$restrict = ".*";

getopts('hr:l:s:c:p:', \%opts);

if (defined($opts{h})) {
    printhelp();
}

if (defined($opts{l})) {
    if ($opts{l} =~ /^(\d+)/) {
        $loopLong = $opts{l};
    }
    else {
        die "ERROR: -l option requires a number!\n";
    }
}

if (defined($opts{s})) {
    if ($opts{s} =~ /^(\d+)/) {
        $loopShort = $opts{s};
    }
    else {
        die "ERROR: -s option requires a number!\n";
    }
}

if (defined($opts{c})) {
    if ($opts{c} =~ /^(\d+):(\d+)/) {
        $cutoffTime = $1 * 60 + $2;
    }
    elsif ($opts{c} =~ /^\d+/) {
        $cutoffTime = $opts{c};
    }
    else {
        die "ERROR: Specify time in -c option with mm:ss or sec!\n";
    }
}

if (defined($opts{r})) {
    $opts{r} =~ s/\s*$//; # Weird - this shouldn't be necessary.
    $restrict = $opts{r};
}

# Try to extract the HVSC_BASE env. var.

$basedir = $ENV{'HVSC_BASE'};
$basedir =~ s/[\/\\]\s*$//; # Removes trailing slash.

if ((!$basedir) or (!-e $basedir)) {
    die "ERROR: Couldn't find/determine HVSC base directory!\n";
}

# Try to find where SIDPLAY is.

$sidplay = $ENV{'SIDPLAY'};

if (!$sidplay) {

    # No SIDPLAY env var, try to find sidplay.

    $sidplay = qx{ which sidplay };
    $sidplay =~ s/\n//g;
    $sidplay = $SIDPLAY if ($sidplay =~ /^no sidplay in/i);

    if (!$sidplay) {
        die "ERROR: Couldn't find SIDPLAY!\n";
    }
    $sidplay = join(' ',$sidplay,$SIDPLAY_OPTIONS);
}

# Try to find where STILVIEW is.

$stilview = $ENV{'STILVIEW'};

if (!$stilview) {

    # No STILVIEW env var, try to find STILView.

    $stilview = qx{ which stilview };
    $stilview =~ s/\n//g;
    $stilview = $STILVIEW if ($stilview =~ /^no stilview in/i);

    if (!$stilview) {
        # This is not an error - STILView is optional.
        print "WARNING: Couldn't find STILVIEW!\n";
    }
}

readSIDList();

if (defined($opts{p})) {
    $opts{p} =~ s/\s*$//; # Weird - this shouldn't be necessary.
    $playlistfile = $opts{p};

    if ($playlistfile =~ /\.spl$/) {

        open (PLAYLIST, "> $playlistfile") || die "ERROR: Couldn't create $playlistfile!\n";

        print PLAYLIST "; Playlist generated by PlayRandom.pl\n";
        print PLAYLIST "; written by LaLa <LaLa\@C64.org\n\n";
        print PLAYLIST "[SIDPLAY/Windows Playlist]\n";
        print PLAYLIST "Format=1\n";
        print PLAYLIST "Order=Normal\n";
        print PLAYLIST "BaseDir=$basedir\n";
        print PLAYLIST "Repeat=No\n\n";

        close PLAYLIST;
    }
    elsif ($playlistfile =~ /\.m3u$/) {

        open (PLAYLIST, "> $playlistfile") || die "ERROR: Couldn't create $playlistfile!\n";

        # No header for Winamp playlists. We just create the file here.

        close PLAYLIST;
    }
    else {
        die "ERROR: Playlist in -p option must have .spl or .m3u extension!\n";
    }
}

$i = 0;
$select = 1;

do {

    if ($select) {
        # Select a random file and a random tune within it.

        $file = $hvscList[rand(@hvscList)];
        if (@{$hvscList{$file}} != 1) {
            $subsong = int(rand(@{$hvscList{$file}})+1);
        }
        else {
            $subsong = 1;
        }

        # Push it onto playlist.
        push (@playlist, "$file:$subsong");
    }
    else {
        ($file, $subsong) = split(/:/,$playlist[$i]);
    }

    $command = "$sidplay -o$subsong $basedir$file";

    # DEBUG
    # print "$command\n";

    # Fancy output.

    print `clear`;

    print "\e[1m\e[7m----------------- PLAYRANDOM.PL ------------------\e[0m\n";
    print "\e[1m(C) 1997-2000 by LaLa <lala\@c64.org>\e[0m\n";
    printspec "PLAYLIST";
    print "HVSC base dir\t: $basedir\n";

    printspec "CURRENT TUNE";
    printf "Now playing\t: %s #%d\n", $file, $subsong;
    printf "\t\t  (%d of %d)\n", $i+1, $#playlist + 1;

    $length = $hvscList{$file}[$subsong-1];
    print  "Length     \t: $length\n";

    if ($length !~ /\(.\)/) {
        $loop = 1;
    }
    else {
        $loop = 0;
    }

    if ($length =~ /^(\d+):(\d+)/) {
        $time = $1 * 60 + $2;
        # Add one sec to non-looping tunes.
        $time++ if (!$loop);
    }
    else {
        # There's no length for this tune.
        # It will play forever.
        $time = -1;
    }

    if ($loop) {
        if ($time <= $cutoffTime) {
            $loopcount = $loopShort;
        }
        else {
            $loopcount = $loopLong;
        }
        print  "Number of loops\t: $loopcount\n";
    }

    # Add tune to playlist.

    if ($playlistfile) {
        open (PLAYLIST, ">> $playlistfile") || die "ERROR: Couldn't access $playlistfile!\n";

        if ($playlistfile =~ /\.spl$/) {
            printf PLAYLIST "[%d]\n", $i+1;
            print PLAYLIST "Filename=$file\n";
            print PLAYLIST "Subsong=$subsong\n";
            if ($time != -1) {
                if ($loop) {
                    printf PLAYLIST "Time=%d\n", $time*$loopcount;
                }
                else {
                    print PLAYLIST "Time=$time\n";
                }
            }
            print PLAYLIST "\n";
        }

        elsif ($playlistfile =~ /\.m3u$/) {
            print PLAYLIST "\"$basedir$file\" $subsong";
            if ($time != -1) {
                if ($loop) {
                    $minutes = ($time*$loopcount)/60;
                    $seconds = ($time*$loopcount)%60;
                }
                else {
                    $minutes = $time/60;
                    $seconds = $time%60;
                }
                printf PLAYLIST " %d:%d", $minutes, $seconds;
            }
            print PLAYLIST "\n";
        }

        close PLAYLIST;
    }

    if ($stilview) {
        my $temp;
        printspec "STIL";
        $temp = `$stilview -e=$file -l=$basedir -b -o`;
        print "---- GLOBAL  COMMENT ----\n$temp" if $temp;
        $temp = `$stilview -e=$file -t=0 -f=comment -l=$basedir -b -s`;
        print "------ TUNE COMMENT -----\n$temp" if $temp;
        $temp = `$stilview -e=$file -t=$subsong -l=$basedir -s -b`;
        print "------ STIL  ENTRY ------\n$temp" if $temp;
        $temp = `$stilview -e=$file -t=$subsong -l=$basedir -s -o`;
        print "------- BUG ENTRY -------\n$temp" if $temp;
    }

    printspec "SIDPLAY";

    # Play the file.

    if ($pid = fork) {
        # Parent

        # Sleep is needed to let SIDPlay dump its output to STDOUT.
        # Otherwise, printouts get messed up.
        sleep(1);

        printspec "EXTRA";

        if ($time != -1) {
            $timeleft = $time-1;
            print "Time left: mm:ss";
            if ($loop) {
                print " (loops left: 001)";
            }
        }
        else {
            print "Hotkeys enabled.\n";
            print "Play (N)ext, (S)ame, or (P)revious tune, (Q)uit\n";
        }

        $done = 0;
        do {
            if ($time == -1) {
                #
                # Input thing...
                #

                $input = <STDIN>;

                # Skip to next tune.
                if ($input =~ /n/i) {
                    $done = 1;
                    if (defined($playlist[$i+1])) {
                        $select = 0;
                    }
                    else {
                        $select = 1;
                    }
                }

                # Play same tune again.
                elsif ($input =~ /s/i) {
                    $done = 1;
                    $i--;
                    if ($i < 0) {$i=-1};
                    $select = 0;
                }

                # Play previous tune.
                elsif ($input =~ /p/i) {
                    $done = 1;
                    $i -= 2;
                    if ($i < -1) {$i=-1};
                    $select = 0;
                }

                # Quit
                elsif ($input =~ /q/i) {
                    kill 9, $pid;
                    exit;
                }

                STDIN->flush();
            }
            else {
                #
                # Time left...
                #

                $min_left = $timeleft/60;
                $sec_left = $timeleft%60;
                if ($loop) {
                    print "\b" x 23;
                }
                else {
                    print "\b" x 5;
                }
                printf "%2d:%02d", $min_left, $sec_left;
                if ($loop) {
                    printf " (loops left: %03d)", $loopcount-1;
                }

                STDOUT->flush();

                if ($timeleft <= 0) {
                    if ($loop) {
                        $loopcount--;
                        if ($loopcount <= 0) {
                            $done = 1;
                        }
                        else {
                            $timeleft = $time;
                        }
                    }
                    else {
                        $done = 1;
                    }
                }
            }

            sleep(1);

            $timeleft-- if ($time != -1);

        } until ($done);

        kill 9, $pid;
    }
    elsif (defined $pid) {
        # Child
        exec "$command";
    }
    else {
        # Weird fork error.
        printspec "ERROR!";
        die "Couldn't fork: $!\n";
    }
    $i++;
} while (1);

